/*****************************************************************************************
*    Analyseur de la teneur de CO2 dans l'air, par caopteur MH-Z19C . Avec indication    *
*de a qualité de l'air par led intelligente WS2812 et détection de présence pour         *
* l'affichage par capteur de distance laser TOF10120 et option de buzzer lorsque la      *
* qualité est mauvaise (option à valider). https://amazing-vacuum-tubes.com              *
*****************************************************************************************/

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#include <Adafruit_NeoPixel.h>
 
#define PIXEL_PIN   8   // Pin de la led RGB
#define PIXEL_COUNT 1   // Nombre de led RGB   

#define SCREEN_WIDTH 128 // Definition des dimentions d'ecran 
#define SCREEN_HEIGHT 64 

#define brightness  50 // luminosité de la Led       

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);
Adafruit_NeoPixel strip = Adafruit_NeoPixel(PIXEL_COUNT, PIXEL_PIN, NEO_GRB + NEO_KHZ800);

const int pwmpin = 7;  // entrée PWM du capteur CO2
int buz = 2;  // sortie d'activation du buzzer (optionelle)
const int range = 5000;
int x_mm; // distance en milimetres
unsigned char i2c_rx_buf[16]; //Reserve un buffer de deux octets pour le bus I2C 
unsigned short lenth_val = 0;  // déclaration et MàZ de la distance mesurée

uint32_t color = 0;

void setup() {

   pinMode(pwmpin, INPUT);  //  DDR des Entrées/Sorties
   pinMode(buz, OUTPUT);
   display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialise l'afficheur I2C adresse 0x3C

   display.clearDisplay();  // Affiche le message d'attente et de présentation
   display.setCursor(0, 0);
   display.setTextColor(WHITE);
   display.setTextSize(2);    
   display.println("B.LECLERC");
   display.setCursor(5, 30);
   display.setTextSize(2);  
   display.println("Chauffage.");
   display.display();
   delay (15000);
}

 void loop() {

 x_mm = ReadDistance(); // Attribut les valeurs buffeurisée lors de la transmition I2C ....

 int ppm_pwm = readCO2PWM(); // Va chercher la valeur donnée par le capteur  
 String IAQ = calc_IAQ(ppm_pwm);



 if (x_mm > 900) {  // Si personne n'est à moins de 80cm extinction de l'afficheur
   display.clearDisplay();  
   display.display();
}
 else {
   display.clearDisplay();  // si qq un est devant, affichage des valeurs capturées
   display.setCursor(0, 0);
   display.setTextSize(2);    
   display.println("CO");
   display.setCursor(23, 8);
   display.setTextSize(1); 
   display.println("2");
   display.setTextSize(2);
   display.setCursor(32, 0);    

   display.setCursor(0, 20);
   display.setTextSize(3);
   display.print(ppm_pwm);
   display.setTextSize(2);
   display.print(" ");
   display.print("ppm");

   display.setCursor(0, 50);
   display.setTextSize(2);
   display.print(IAQ);
  
   display.display();
   delay(5000); 
  }

  int temp = 150 - (150 * ppm_pwm / 1800);  // Calcul de la teneur en fonction de la durée de l'impulsion PWM
  if (temp < 0)temp += 256;
  int x=100;               
  color = Wheel(temp, brightness , brightness , brightness );  // Valeur de couleur et luminosité de la LED
  strip.begin();                            // Initialise La LED WS2812
  strip.setPixelColor(0, color);            
  strip.show();                             // Envoi des données à la RGB LED
    

}

int readCO2PWM() {  // Lecture de la durée de la largeur de pulse PWM (rapport cyclique)

  unsigned long th;
  int ppm_pwm = 0;
 float pulsepercent;

    do {

    th = pulseIn(pwmpin, HIGH, 1004000) / 1000;
    float pulsepercent = th / 1004.0;
    ppm_pwm = range * pulsepercent;

     } while (th == 0);

  return ppm_pwm; 
}

uint32_t Wheel(byte value, byte Rdimm, byte Gdimm, byte Bdimm) // Mixage de la couleur en fonction de la teneur en CO2
{
  uint16_t Phase, Blue, Green, Red, Level;
  Phase = value / 21;  // Pour définir le cas de la fonction switch
  Level = value % 21;

  switch ( Phase )
  {
    case 0:                 //Rouge >1548
      {
        Red   = 100;
        Green = 0;
        Blue  = 0;
        //digitalWrite(buz, HIGH); // Si option Buzzer 
        //delay(5000);
        //digitalWrite(buz, LOW);
      }
      break;

    case 1:                 //Orange > 1296
      {
        Red   = 100;
        Green = 22;
        Blue  = 0;
      }
      break;

    case 2:                 //Jaune > 1044
      {
        Red   = 100;
        Green = 100;
        Blue  = 0;
      }
      break;

    case 3:                 //Vertjaune >792
      {
        Red   = 75;
        Green = 100;
        Blue  = 0;
      }
      break;

    case 4:                 //Vert >540
      {
        Red   = 0;
        Green = 100;
        Blue  = 0;
      }
      break;
    case 5:                 //bleu >288
      {
        Red   = 0;
        Green = 0;
        Blue  = 100;
      }
      break;
    default:
      Red = Blue = 255; Green =0; // Mauve
  }
  return strip.Color(Red * Rdimm / 100, Green * Gdimm / 100, Blue * Bdimm / 100);
}

String calc_IAQ(int ppm_pwm) { // Défini l'Indice de Qualité de l'Air (IAQ) suivant le PWM reçu et calculé
  String IAQ;
  if      (ppm_pwm > 1500) IAQ += "Aeration !";
  else if (ppm_pwm >= 1250 && ppm_pwm <= 1500) IAQ += "Mauvais ";
  else if (ppm_pwm  > 950 && ppm_pwm <= 1250) IAQ += "Moyen";
  else if (ppm_pwm  > 650 && ppm_pwm <= 950) IAQ += "Bon Air";
  else if (ppm_pwm  > 400 && ppm_pwm <= 650) IAQ += "Air Pur";
   return IAQ;
}


void SensorRead(unsigned char addr,unsigned char* datbuf,unsigned char cnt) 
{
  unsigned short result=0;
  // étape 1 : demander au capteur de lire les échos
  Wire.beginTransmission(82); // transmettre au composant #82 (0x52)
  //l'adresse indiquée dans la fiche technique est 164 (0xa4) mais l'adressage i2c utilise les 7 bits de poids fort donc c'est 82
  Wire.write(byte(addr));      // définit l'adresse de données de distance (addr)
  Wire.endTransmission();      // Fin de transmission

  // étape 2 : attendez que les lectures se produisent
  delay(1); // la fiche technique suggère au moins 30µS
  
  // étape 3 : demande de la lecture du capteur
  Wire.requestFrom(82, cnt);    //demande des octets contenus dans le composant esclave #82 (0x52)
  
  // étape 4 : reçoit la lecture du capteur
  if (cnt <= Wire.available()) { // si deux octets ont été reçus, on reçoit l'octet de poids fort (écrase la lecture précédente)puis l'octet de poids faible
    *datbuf++ = Wire.read();
    *datbuf++ = Wire.read(); 
  }
}

   int ReadDistance(){ // Lecture de la distance et mémorisation des octets reçus dans les deux buffers
    SensorRead(0x00,i2c_rx_buf,2);
    lenth_val=i2c_rx_buf[0];
    lenth_val=lenth_val<<8;
    lenth_val|=i2c_rx_buf[1];
    delay(300); 
    return lenth_val;
}


